@@ -123,6 +123,8 @@ enum {
 	REQUEST_STARTED,
 	REQUEST_UNQUEUED,
 	AUTHENTICATE,
+	CONNECTION_CREATED,
+	TUNNELING,
 	LAST_SIGNAL
 };
 
@@ -380,6 +382,27 @@ soup_session_class_init (SoupSessionClass *session_class)
 			      SOUP_TYPE_AUTH,
 			      G_TYPE_BOOLEAN);
 
+	signals[CONNECTION_CREATED] =
+		g_signal_new ("connection-created",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      0,
+			      NULL, NULL,
+			      soup_marshal_NONE__OBJECT,
+			      G_TYPE_NONE, 1,
+			      SOUP_TYPE_CONNECTION);
+
+	signals[TUNNELING] =
+		g_signal_new ("tunneling",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      0,
+			      NULL, NULL,
+			      soup_marshal_NONE__OBJECT,
+			      G_TYPE_NONE, 1,
+			      SOUP_TYPE_CONNECTION);
+
+
 	/* properties */
 	/**
 	 * SOUP_SESSION_PROXY_URI:
@@ -809,16 +832,12 @@ soup_session_host_new (SoupSession *session, SoupURI *uri)
 	return host;
 }
 
-/* Note: get_host_for_message doesn't lock the host_lock. The caller
- * must do it itself if there's a chance the host doesn't already
- * exist.
- */
+/* Requires host_lock to be locked */
 static SoupSessionHost *
-get_host_for_message (SoupSession *session, SoupMessage *msg)
+get_host_for_uri (SoupSession *session, SoupURI *uri)
 {
 	SoupSessionPrivate *priv = SOUP_SESSION_GET_PRIVATE (session);
 	SoupSessionHost *host;
-	SoupURI *uri = soup_message_get_uri (msg);
 
 	host = g_hash_table_lookup (priv->hosts, uri);
 	if (host)
@@ -830,6 +849,16 @@ get_host_for_message (SoupSession *session, SoupMessage *msg)
 	return host;
 }
 
+/* Note: get_host_for_message doesn't lock the host_lock. The caller
+ * must do it itself if there's a chance the host doesn't already
+ * exist.
+ */
+static SoupSessionHost *
+get_host_for_message (SoupSession *session, SoupMessage *msg)
+{
+	return get_host_for_uri (session, soup_message_get_uri (msg));
+}
+
 static void
 free_host (SoupSessionHost *host)
 {
@@ -882,10 +911,12 @@ redirect_handler (SoupMessage *msg, gpointer user_data)
 	     !SOUP_METHOD_IS_SAFE (msg->method)) ||
 	    (msg->status_code == SOUP_STATUS_MOVED_PERMANENTLY &&
 	     msg->method == SOUP_METHOD_POST)) {
-		/* Redirect using a GET */
-		g_object_set (msg,
-			      SOUP_MESSAGE_METHOD, SOUP_METHOD_GET,
-			      NULL);
+		if (msg->method != SOUP_METHOD_HEAD) {
+			/* Redirect using a GET */
+			g_object_set (msg,
+				      SOUP_MESSAGE_METHOD, SOUP_METHOD_GET,
+				      NULL);
+		}
 		soup_message_set_request (msg, NULL,
 					  SOUP_MEMORY_STATIC, NULL, 0);
 		soup_message_headers_set_encoding (msg->request_headers,
@@ -1046,8 +1077,10 @@ soup_session_connection_failed (SoupSession *session,
 }
 
 static void
-tunnel_connected (SoupSession *session, SoupMessage *msg, gpointer user_data)
+tunnel_connected (SoupMessage *msg, gpointer user_data)
 {
+	SoupSession *session = user_data;
+
 	if (SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
 		SoupSessionPrivate *priv = SOUP_SESSION_GET_PRIVATE (session);
 		SoupMessageQueueItem *item =
@@ -1059,6 +1092,7 @@ tunnel_connected (SoupSession *session, SoupMessage *msg, gpointer user_data)
 		g_object_set (item->conn,
 			      SOUP_CONNECTION_PROXY_URI, NULL,
 			      NULL);
+		soup_message_queue_item_unref (item);
 	}
 }
 
@@ -1082,9 +1116,13 @@ soup_session_make_connect_message (SoupSession *session,
 
 	/* Call the base implementation of soup_session_queue_message
 	 * directly, to add msg to the SoupMessageQueue and cause all
-	 * the right signals to be emitted.
+	 * the right signals to be emitted. We can't use
+	 * queue_message's callback arg in this case because that's
+	 * actually implemented by the subclasses.
 	 */
-	queue_message (session, msg, tunnel_connected, NULL);
+	g_signal_connect (msg, "finished",
+			  G_CALLBACK (tunnel_connected), session);
+	queue_message (session, msg, NULL, NULL);
 	item = soup_message_queue_lookup (priv->queue, msg);
 	g_object_unref (msg);
 	return item;
@@ -1202,6 +1240,8 @@ soup_session_get_connection (SoupSession *session,
 			  G_CALLBACK (connection_disconnected),
 			  session);
 
+	g_signal_emit (session, signals[CONNECTION_CREATED], 0, conn);
+
 	g_hash_table_insert (priv->conns, conn, host);
 
 	priv->num_conns++;
@@ -1512,6 +1552,38 @@ soup_session_abort (SoupSession *session)
 	g_slist_free (conns);
 }
 
+/**
+* soup_session_prepare_for_uri:
+* @session: a #SoupSession
+* @uri: a #SoupURI which may be required
+*
+* Tells @session that @uri may be requested shortly, and so the
+* session can try to prepare (resolving the domain name, obtaining
+* proxy address, etc.) in order to work more quickly once the URI is
+* actually requested.
+*
+* This method acts asynchronously, in @session's %async_context.
+* If you are using #SoupSessionSync and do not have a main loop running,
+* then you can't use this method.
+*
+* Since: 2.30
+**/
+void
+soup_session_prepare_for_uri (SoupSession *session, SoupURI *uri)
+{
+	SoupSessionPrivate *priv = SOUP_SESSION_GET_PRIVATE (session);
+	SoupSessionHost *host;
+	SoupAddress *addr;
+
+	g_mutex_lock (priv->host_lock);
+	host = get_host_for_uri (session, uri);
+	addr = g_object_ref (host->addr);
+	g_mutex_unlock (priv->host_lock);
+
+	soup_address_resolve_async (addr, priv->async_context,
+				    NULL, NULL, NULL);
+}
+
 /**
  * soup_session_add_feature:
  * @session: a #SoupSession
